//
// NAME: CIDGui_PaintBrush.Hpp
//
// DESCRIPTION:
//
//  This is the header for the CIDGui_PaintBrush.Cpp module. This module
//  provides the PAINTBRUSH class, which is the means by which graphical
//  output is accomplished to windows. When a new PAINTBRUSH object is
//  created, the caller must pass a window object. The window object is used
//  to create a PS for the window. This class provides a special constructor
//  for the WINDOW class, which will create a PAINTBRUSH object from the
//  WM_PAINT message handler. This paint brush object will be passed to the
//  window in the bPaint() method.
//
//
//  The class hierarchy here is:
//
//              CIDOBJECT
//                  
//              PAINTBRUSH
//
//
// AUTHOR: Dean Roddey
//
// CREATE DATE: 01/22/94
//
// COPYRIGHT: 1992..1994, 'CIDCorp
//
// CAVEATS/GOTCHAS:
//
//  1)  Paint brush objects should not be created too loosely. They represent
//      system resources that are limited. They should used and destroyed as
//      soon as possible.
//
//  2)  At this point, there is no way to create PS's in the way that you
//      can via GpiCreatePS() and set up different metrics and stuff. That
//      functionality has not been needed yet but will be added soon.
//
// MODIFICATION LOG:
//

// Avoid multiple inclusions
#if            !defined(CIDGUI_PAINTBRUSH_HPP)
#define        CIDGUI_PAINTBRUSH_HPP


// -----------------------------------------------------------------------------
//   CLASS: PAINTBRUSH
//  PREFIX: ptb
// -----------------------------------------------------------------------------
class               PAINTBRUSH : public CIDOBJECT
{
    public          :
        // ---------------------------------------------------------------------
        //  Constructors and Destructors.
        //
        //  PAINTBRUSH
        //      The default constructor is hidden. The primary version accepts
        //      the window object to draw on.
        // ---------------------------------------------------------------------
        PAINTBRUSH
        (
                    WINDOW&                 wndParent
        );

        ~PAINTBRUSH();


        // ---------------------------------------------------------------------
        //  Public, inherited, virtual methods
        //
        //  FormatToStr
        //      Formats some debug info about this paintbrush to the string
        //      buffer.
        // ---------------------------------------------------------------------
        STDVPUBMETHODS(PAINTBRUSH,CIDOBJECT,tCIDLib::eFALSE);

        virtual tCIDLib::VOID FormatToStr
        (
                    STRGBUF&                strbDest
        )   const;


        // ---------------------------------------------------------------------
        //  Public, non-virtual methods
        //
        //  operator HPS
        //      An implicit conversion so that PAINTBRUSH objects can be
        //      passed directly to GPI APIs. The PS is extracted.
        //
        //  areaString
        //      This method will return a 0 based AREA object that will contain
        //      the passed string using the current font. There is a second
        //      version that will do substrings of the whole string. This is
        //      very convienent for things like underlining certain chars.
        //
        //  Box
        //      There are three versions. One draws the current position to
        //      the passed point, another from two individual points
        //      representing diagonal corners of the box, and another from
        //      a passed AREA object. The rounding value controls how deeply
        //      rounded the corners are. It should be less than 1/2 the
        //      shortest side of the box to be useful.
        //
        //  c4EnumFonts
        //      This method will return a linked list FONTINFO fonts for the
        //      available fonts that match the passed criteria. It is based on
        //      the _c4EnumFonts() intrafacility method from CIDGui_Fonts.Cpp.
        //
        //  CreateFont
        //      This method will create a font using the passed font info and
        //      give it the passed name.
        //
        //  DisplayText
        //      There are a number of versions. The most basic just takes a
        //      string and draws it at the current position. Another draws the
        //      string at an indicated position. And the third provides a set
        //      of flags that support a number of display options. It also draws
        //      the text clipped and justified with a passed area.
        //
        //  DrawBorder
        //      This method will draw the indicated type of border.
        //
        //  eBgnMixMode
        //      This method will set/get the background mix mode for this
        //      window.
        //
        //  eFgnMixMode
        //      This method will set/get the foreground mix mode for this
        //      window.
        //
        //  Ellipse
        //      This method will draw an ellipse. There is one version that
        //      uses a center point and radius and other version that will draw
        //      an ellipse inside a passed area.
        //
        //  eLineStyle
        //      This method will set/get the line style for this window.
        //
        //  ePattern
        //      This method will set/get the pattern type for this window
        //
        //  FillCurPath
        //      This method will fill the current path, either with the passed
        //      color in one version or the current foreground color in the
        //      other.
        //
        //  finfCurrent
        //      This method will return a FONTINFO object that contains the
        //      info on the font currently installed in this window.
        //
        //  Line
        //      There are two versions. One draws from the current position to
        //      the indicated position, and the other provides starting and
        //      ending positions. In both cases, the current position ends up
        //      at the ending line point.
        //
        //  pntCurPos
        //      This method will get the current position.
        //
        //  pntRelativePos
        //      This method will move the current position by the indicated
        //      offsets. It will return the new position.
        //
        //  PolyLine
        //  PolyLineDisjoint
        //      These methods will draw a list of line segments that are
        //      represented by an array of POINT objects. PolyLine draws a
        //      contiguous line for one point to the next. PolyLineDisjoint
        //      draws line segments represented by each two points. A move is
        //      done to the even point and a line drawn to the odd point.
        //
        //  ResetAttrs
        //      This method will reset the graphical attributes of the paint
        //      brush object.
        //
        //  ResetClipPath
        //      This method will reset the clip path to infinity.
        //
        //  rgbBgnColor
        //  rgbFgnColor
        //      This methods return the current foreground and background
        //      color.
        //
        //  SetBgnColor
        //  SetFgnColor
        //      These methods set the foreground and background color.
        //
        //  ScrollArea
        //      This method will scroll the passed area up, down, left, or
        //      right by the indicated number of pels. It can optionally
        //      invalidate the uncovered area. There is a simple version and
        //      another that allows a clip area and returns the invalidated
        //      area.
        //
        //  SetClipPath
        //      This method will set the clip to the passed path. The new path
        //      is combined with the existing clip path according to the
        //      passed clip options. The path can be an area, two points that
        //      define an area, or the existing path which was set via SetPath.
        //
        //  SetCurFont
        //      This method will set the current font of this window. The caller
        //      passes the name of the font, which was set when the font was
        //      created via CreateFont.
        //
        //  SetCurPos
        //      This method will set the current position.
        //
        //  SetDefaultFont
        //      This method will set the current font to the default system
        //      font.
        //
        //  SetPath
        //      This method will set the current path. It will accept either
        //      an area or a container of POINT objects that define a path.
        //
        //  SetPel
        //      Sets the indicated pixel to the passed color
        //
        //  StringCharOfs
        //      This method will return the x offsets (from the origin of the
        //      string) to the start and end of a particular character.
        //
        //  Stroke
        //      This method will stroke a line around the area that is of the
        //      indicated thickness. The rounding parm controls how deeply
        //      rounded the corners are. It should be less than 1/2 the size of
        //      the shorted dimension of the area to be useful.
        //      There is a second version that will stroke a circle with the
        //      given x and y radii and origin.
        // ---------------------------------------------------------------------
        operator HPS
        (
        )   const   {return __hPS;}

        AREA areaString
        (
            const   STRGBUF&                strbText
        )   const;

        AREA areaString
        (
            const   STRGBUF&                strbText
            ,       tCIDLib::CARD4          c4Start
            ,       tCIDLib::CARD4          c4End
        )   const;

        tCIDLib::VOID Box
        (
            const   POINT&                  pntTo
            ,       tCIDLib::CARD4          c4Rounding = 0
        );

        tCIDLib::VOID Box
        (
            const   POINT&                  pntFrom
            , const POINT&                  pntTo
            ,       tCIDLib::CARD4          c4Rounding = 0
        );

        tCIDLib::VOID Box
        (
            const   AREA&                   areaBox
            ,       tCIDLib::CARD4          c4Rounding = 0
        );

        tCIDLib::CARD4 c4EnumFonts
        (
            const   STRGBUF&                strbFaceName
            ,       tCIDGui::eFONTFLAGS     eSelection
            ,       tCIDGui::eFONTFLAGS     eSignificant
            ,       LINKLIST&               llstTarget
        );

        tCIDLib::VOID CreateFont
        (
            const   FONTINFO&               finfNew
            ,       tCIDGui::eFATTRS        eFAttrs
            , const STRG16&                 strName
        );

        tCIDLib::VOID DisplayText
        (
            const   STRGBUF&                strbText
        );

        tCIDLib::VOID DisplayText
        (
            const   STRGBUF&                strbText
            , const POINT&                  pntPos
        );

        tCIDLib::VOID DisplayText
        (
            const   STRGBUF&                strbText
            , const AREA&                   areaClip
            ,       tCIDLib::eHJUSTIFY      eHJustify = tCIDLib::eHJUSTIFY_LEFT
            ,       tCIDLib::eVJUSTIFY      eVJustify = tCIDLib::eVJUSTIFY_BOTTOM
            ,       tCIDGui::eTEXTFLAGS     eFlags = tCIDGui::eTEXTFLAG_NONE
        );

        tCIDLib::VOID DrawBorder
        (
                    tCIDGui::eBORDERS       eBorder
            , const AREA&                   areaPos
            ,       tCIDLib::CARD4          c4Width
        );

        tCIDGui::eMIXMODES eBgnMixMode() const;

        tCIDGui::eMIXMODES eBgnMixMode
        (
                    tCIDGui::eMIXMODES      eNewMixMode
        );

        tCIDGui::eMIXMODES eFgnMixMode
        (
        )   const;

        tCIDGui::eMIXMODES eFgnMixMode
        (
                    tCIDGui::eMIXMODES      eNewMixMode
        );

        tCIDLib::VOID Ellipse
        (
            const   POINT&                  pntCenter
            ,       tCIDLib::CARD4          c4Radius
        );

        tCIDLib::VOID Ellipse
        (
            const   AREA&                   areaCircle
        );

        tCIDGui::eLINESTYLES eLineStyle() const;

        tCIDGui::eLINESTYLES eLineStyle
        (
                    tCIDGui::eLINESTYLES    eLineStyle
        )  const;

        tCIDGui::ePATTERNS ePattern
        (
        )   const;

        tCIDGui::ePATTERNS ePattern
        (
        
                    tCIDGui::ePATTERNS      ePattern
        );

        tCIDLib::VOID FillCurPath
        (
                    tCIDGui::ePATHFILL      eFill = tCIDGui::ePATHFILL_ALTERNATE
        )           {FillCurPath(rgbFgnColor(), eFill);}

        tCIDLib::VOID FillCurPath
        (
            const   RGBCLR&                 rgbFillClr
            ,       tCIDGui::ePATHFILL      eFill = tCIDGui::ePATHFILL_ALTERNATE
        );

        FONTINFO finfCurrent() const;

        tCIDLib::VOID Line
        (
            const   POINT&                  pntTo
        );

        tCIDLib::VOID Line
        (
            const   POINT&                  pntFrom
            , const POINT&                  pntTo
        );

        POINT pntCurPos() const;

        POINT pntRelativePos
        (
                    tCIDLib::INT4           i4XOfs
            ,       tCIDLib::INT4           i4YOfs
        );

        tCIDLib::VOID PolyLine
        (
                    POINT*                  apntList
            ,       tCIDLib::CARD4          c4Pnts
        );

        tCIDLib::VOID PolyLineDisjoint
        (
                    POINT*                  apntList
            ,       tCIDLib::CARD4          c4Segs
        );

        tCIDLib::VOID ResetAttrs();

        tCIDLib::VOID ResetClipPath();

        RGBCLR rgbBgnColor() const;

        RGBCLR rgbFgnColor() const;

        tCIDLib::VOID SetBgnColor
        (
            const   RGBCLR&                 rgbNew
        );

        tCIDLib::VOID SetFgnColor
        (
            const   RGBCLR&                 rgbNew
        );

        tCIDLib::VOID SetClipPath
        (
                    tCIDGui::ePATHFILL      eFillType
            ,       tCIDGui::eCLIPOPTS      eClipOpts
        );

        tCIDLib::VOID SetClipPath
        (
            const   AREA&                   areaToClip
            ,       tCIDGui::ePATHFILL      eFillType
            ,       tCIDGui::eCLIPOPTS      eClipOpts
        );

        tCIDLib::VOID SetClipPath
        (
            const   POINT&                  pntFrom
            , const POINT&                  pntTo
            ,       tCIDGui::ePATHFILL      eFillType
            ,       tCIDGui::eCLIPOPTS      eClipOpts
        );

        tCIDLib::VOID SetCurFont
        (
            const   STRG16&                 strName
        );

        tCIDLib::VOID SetCurPos
        (
            const   POINT&                  pntNew
        );

        tCIDLib::VOID SetCurPos
        (
                    tCIDLib::INT4           i4XPos
            ,       tCIDLib::INT4           i4YPos
        );

        tCIDLib::VOID SetDefaultFont();

        tCIDLib::VOID SetPath
        (
            const   AREA&                   areaPath
        );

        tCIDLib::VOID SetPath
        (
            const   CONTAINER&              cntPoints
        );

        tCIDLib::VOID SetPel
        (
                    tCIDLib::INT4           i4XPos
            ,       tCIDLib::INT4           i4YPos
            , const RGBCLR&                 rgbClr
        );

        tCIDLib::VOID SetPel
        (
            const   POINT&                  pntPos
            , const RGBCLR&                 rgbClr
        );

        tCIDLib::VOID StringCharOfs
        (
            const   STRGBUF&                strbText
            ,       tCIDLib::CARD4          c4Ind
            ,       tCIDLib::INT4&          i4XStart
            ,       tCIDLib::INT4&          i4XEnd
        );

        tCIDLib::VOID Stroke
        (
            const   AREA&                   areaTarget
            ,       tCIDLib::CARD4          c4Width = 1
            ,       tCIDLib::CARD4          c4Rounding = 0
        );

        tCIDLib::VOID Stroke
        (
            const   POINT&                  pntOrg
            ,       tCIDLib::CARD4          c4Width
            ,       tCIDLib::CARD4          c4XRadius
            ,       tCIDLib::CARD4          c4YRadius
        );


    protected       :
        // ---------------------------------------------------------------------
        //  Declare our friends
        // ---------------------------------------------------------------------
        friend      class WINDOW;
        friend      class GRAPHATTR;
        friend      class BACKEDWND;


        // ---------------------------------------------------------------------
        //  Protected data types
        //
        //  ePSTYPE
        //      This indicates the type of the PS, which controls the action
        //      taken by the destructor.
        // ---------------------------------------------------------------------
        enum            ePSTYPE
        {
            ePS_PAINT           // Gotten via WinBeginPaint
            , ePS_TEMPORARY     // Gotten via WinGetPS
            , ePS_PERSISTENT    // Gotten vai GpiCreatePS
        };

        // ---------------------------------------------------------------------
        //  Protected, non-virtual methods
        //
        //  _Draw3DBorder
        //      An efficient method for drawing raised/sunken borders. It is
        //      primarily for DrawBorder() to call. areaPos is adjusted by the
        //      border width.
        //
        //  PAINTBRUSH
        //      The default constructor must be hidden so that only derived
        //      classes have access to it. A special version is available to
        //      our friend WINDOW, so he can create a paint brush during
        //      WM_PAINT messages.
        //
        //  _plnodFindFnt
        //      This is used internally to find a particular font in the
        //      __llstFonts list, by name or by font id. It returns a pointer
        //      to the list node object for that font, or 0 if no match.
        // ---------------------------------------------------------------------
        tCIDLib::VOID _Draw3DBorder
        (
                    tCIDLib::eBOOL          bRaised
            ,       AREA&                   areaPos
            ,       tCIDLib::INT4           i4Width
            , const RGBCLR&                 rgbBgn
            , const RGBCLR&                 rgbFgn
        );

        PAINTBRUSH();

        PAINTBRUSH
        (
                    HPS                     hPS
            ,       WINDOW&                 wndTarget
            ,       ePSTYPE                 eType
        );

        LLSTNODE* _plnodFindFnt
        (
            const   STRG16&                 strName
        );

        LLSTNODE* _plnodFindFnt
        (
                    tCIDLib::INT4           i4FontId
        );


    private         :
        // ---------------------------------------------------------------------
        //  Private data members
        //
        //  __eType
        //      This is set by the contructor to indicate what type of
        //      PS we have.
        //
        //  __hPS
        //      This is the PS that we are controlling with this paint brush
        //      object.
        //
        //  __llstFonts
        //      This linked list is used to store a list of FONT objects. The
        //      FONT class is private to CIDGui_GpiWindow.Cpp.
        //
        //  __wndTarget
        //      This is the window that we are painting into.
        // ---------------------------------------------------------------------
        ePSTYPE                 __eType;
        HPS                     __hPS;
        LINKLIST                __llstFonts;
        WINDOW&                 __wndTarget;
};

#define NUL_PAINTBRUSH          (*(PAINTBRUSH*)0)



//
//  Some global formatting functions to format our enums into a string buffer
//  or text file object. The functions are in CIDGui_FmtType.Cpp.
//
STRGBUF& operator<<(STRGBUF& strbDest, tCIDGui::eMIXMODES);
OTXTFILE& operator<<(OTXTFILE& otfDest, tCIDGui::eMIXMODES);
STRGBUF& operator<<(STRGBUF& strbDest, tCIDGui::eLINESTYLES);
OTXTFILE& operator<<(OTXTFILE& otfDest, tCIDGui::eLINESTYLES);
STRGBUF& operator<<(STRGBUF& strbDest, tCIDGui::ePATTERNS);
OTXTFILE& operator<<(OTXTFILE& otfDest, tCIDGui::ePATTERNS);

#endif
